/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "uitextbox.h"
#include "util/memory.h"
#include "assert/assert.h"
#include "input.h"

uitextbox_t UI_TEXTBOX;

void uiTextboxInit() {
  memoryZero(&UI_TEXTBOX, sizeof(uitextbox_t));
}

void uiTextboxUpdate() {
  if(UI_TEXTBOX.visible == false) return;

  if(UI_TEXTBOX.charsRevealed < UI_TEXTBOX.pageChars[UI_TEXTBOX.page]) {
    if((RENDER_FRAME % 4) == 0) {
      // printf("Revealing char\n");
      UI_TEXTBOX.charsRevealed++;
    } else if(inputIsDown(INPUT_BIND_ACTION)) {
      UI_TEXTBOX.charsRevealed++;
    }

  } else {
    if(inputPressed(INPUT_BIND_ACTION)) {
      if(UI_TEXTBOX.page <= UI_TEXTBOX.pageCount - 1) {
        uitextbox_t *textbox = &UI_TEXTBOX;
        printf("next page?\n");
        UI_TEXTBOX.page++;
        UI_TEXTBOX.charsRevealed = 0;
      } else {
        printf("Closing textbox\n");
        // Close the textbox
        UI_TEXTBOX.visible = false;
        UI_TEXTBOX.page = 0;
        UI_TEXTBOX.charsRevealed = 0;
      }
    }
  }
}

void uiTextboxSetText(const char_t *text) {
  assertNotNull(text, "Text pointer cannot be NULL, call uiTextboxClose()");
  
  memoryZero(UI_TEXTBOX.text, sizeof(UI_TEXTBOX.text));
  memoryZero(UI_TEXTBOX.lineLengths, sizeof(UI_TEXTBOX.lineLengths));
  memoryZero(UI_TEXTBOX.pageChars, sizeof(UI_TEXTBOX.pageChars));
  UI_TEXTBOX.pageCount = 0;
  UI_TEXTBOX.totalChars = 0;
  UI_TEXTBOX.page = 0;
  UI_TEXTBOX.charsRevealed = 0;
  UI_TEXTBOX.visible = true;
  
  char_t c;
  uint16_t i = 0;
  uint16_t j = 0;
  uint8_t line = 0;
  while((c = text[i++]) != '\0') {
    assertTrue(c != '\r', "Carriage return characters not allowed.");
    assertTrue(c != '\t', "Tab characters not allowed.");

    if(c == '\n') {
      goto newline;
    } else {
      goto character;
    }

    newline: {
      line++;
      if(line >= UI_TEXTBOX_LINE_COUNT) {
        goto newPage;
      }
      continue;
    }

    character: {
      UI_TEXTBOX.text[j] = c;
      UI_TEXTBOX.totalChars++;
      UI_TEXTBOX.lineLengths[line]++;
      UI_TEXTBOX.pageChars[UI_TEXTBOX.pageCount]++;
    }

    newPage: {
      assertTrue(
        UI_TEXTBOX.pageCount < UI_TEXTBOX_PAGE_COUNT_MAX,
        "Exceeded maximum number of pages in textbox."
      );
      UI_TEXTBOX.pageCount++;
      line = 0;
      continue;
      // j = 0; // Reset character index for new page
    }

    assertUnreachable("Code should not reach here, all cases handled.");
  }




  // Hard coded testing values
  // char_t buffer[UI_TEXTBOX_CHARS_PER_LINE];
  // uint8_t line;

  // for(line = 0; line < UI_TEXTBOX_LINE_COUNT; line++) {
  //   // Set the text for each line
  //   sprintf(buffer, "HC Line %d", line);
  //   UI_TEXTBOX.lineLengths[line] = strlen(text); // Set line length for each line
  //   UI_TEXTBOX.totalChars += UI_TEXTBOX.lineLengths[line];
  //   UI_TEXTBOX.pageChars[UI_TEXTBOX.pageCount] += UI_TEXTBOX.lineLengths[line];
    
  //   memoryCopy(
  //     UI_TEXTBOX.text + line * (UI_TEXTBOX_CHARS_PER_LINE),
  //     buffer,
  //     UI_TEXTBOX.lineLengths[line]
  //   );

  //   if(line == 3 || line == 7 || line == 11 || line == 15) {
  //     // Increment page count every 4 lines
  //     UI_TEXTBOX.pageCount++;
  //   }
  // }
}